package com.miteksystems.misnapextractioncontroller;

import android.content.Context;
import android.content.pm.ActivityInfo;
import android.graphics.Bitmap;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.VisibleForTesting;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;

import com.miteksystems.imaging.JPEGProcessor;
import com.miteksystems.imaging.PreviewFrameConverter;
import com.miteksystems.misnap.ICamera;
import com.miteksystems.misnap.IFrameHandler;
import com.miteksystems.misnap.analyzer.IAnalyzeResponse;
import com.miteksystems.misnap.analyzer.MiSnapAnalyzer;
import com.miteksystems.misnap.analyzer.MiSnapAnalyzerResult;
import com.miteksystems.misnap.analyzer.MiSnapAnalyzerResultsProcessor;
import com.miteksystems.misnap.detector.MiSnapDocumentDetector;
import com.miteksystems.misnap.detector.MiSnapDocumentDetectorFactory;
import com.miteksystems.misnap.documents.BaseDocument;
import com.miteksystems.misnap.documents.BasicMrz;
import com.miteksystems.misnap.documents.ExtendedMrz;
import com.miteksystems.misnap.mibidata.MibiData;
import com.miteksystems.misnap.natives.HaarCascade;
import com.miteksystems.misnap.natives.MiSnapScience;
import com.miteksystems.misnap.params.CameraParamMgr;
import com.miteksystems.misnap.params.ScienceApi;
import com.miteksystems.misnap.params.ScienceParamMgr;
import com.miteksystems.misnap.params.UxpConstants;
import com.miteksystems.misnap.utils.Utils;

import org.greenrobot.eventbus.EventBus;
import org.json.JSONObject;

import java.lang.ref.WeakReference;
import java.nio.ByteBuffer;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.RejectedExecutionException;

public class MiSnapExtractionController implements IFrameHandler {
    private static final String TAG = MiSnapExtractionController.class.getName();
    private WeakReference<Context> weakActivityContext;
    private ICamera camera;
    private MiSnapAnalyzer cornerAnalyzer;
    private MiSnapAnalyzer mrz3LineAnalyzer;
    private MiSnapAnalyzer mrz1LineAnalyzer;
    private MiSnapDocumentDetector documentDetector;
    private CameraParamMgr cameraParamMgr;
    private ScienceParamMgr scienceParamMgr;
    private ExecutorService executorService;
    private volatile boolean analyzingInProgress = false;
    private volatile boolean executorReady = true;
    private static long DELAY_BEFORE_ALLOWING_SNAP_IN_MS = 1000;
    private long lastGoodSnapTimeInMs;
    private BaseDocument lastExtractedValue = null;
    private MutableLiveData<MiSnapExtractionControllerResult> result;

    private int detectorOrientation;

    public MiSnapExtractionController(@NonNull Context activityContext,
                                      @NonNull ICamera camera,
                                      MiSnapAnalyzer cornerAnalyzer, MiSnapAnalyzer mrz3LineAnalyzer,
                                      MiSnapAnalyzer mrz1LineAnalyzer,
                                      MiSnapDocumentDetector documentDetector, JSONObject settings) {
        this.weakActivityContext = new WeakReference<>(activityContext);
        this.camera = camera;
        this.cornerAnalyzer = cornerAnalyzer;
        this.mrz3LineAnalyzer = mrz3LineAnalyzer;
        this.mrz1LineAnalyzer = mrz1LineAnalyzer;
        this.documentDetector = documentDetector;
        this.cameraParamMgr = new CameraParamMgr(settings);
        this.scienceParamMgr = new ScienceParamMgr(settings);
        this.executorService = Executors.newSingleThreadExecutor();
        this.result = new MutableLiveData<>();
    }

    @VisibleForTesting
    public MiSnapExtractionController(@NonNull Context appContext,
                                      @NonNull ICamera camera,
                                      MiSnapAnalyzer cornerAnalyzer, MiSnapAnalyzer mrz3LineAnalyzer,
                                      MiSnapAnalyzer mrz1LineAnalyzer,
                                      MiSnapDocumentDetector documentDetector, JSONObject settings,
                                      ExecutorService executorService) {
        this(appContext, camera, cornerAnalyzer, mrz3LineAnalyzer, mrz1LineAnalyzer, documentDetector, settings);
        this.executorService = executorService;
    }

    public void start() {
        camera.addFrameHandler(this);
    }

    public void end() {
        if (cornerAnalyzer != null) {
            cornerAnalyzer.deinit();
        }

        if (mrz3LineAnalyzer != null) {
            mrz3LineAnalyzer.deinit();
        }

        executorReady = false;
        executorService.shutdownNow();

        lastExtractedValue = null;
    }

    @Override
    public void handlePreviewFrame(final byte[] frame, final int width, final int height,
                                   final int colorSpace, final int deviceOrientation, final int cameraRotationDegrees) {
        if (!analyzingInProgress) {
            try {
                executorService.submit(() -> {
                    try {
                        analyzingInProgress = true;
                        Context activityContext = weakActivityContext.get();
                        if (activityContext != null) {
                            // Can't trust any single value, so need to constantly reset the orientation
                            int documentOrientation = OrientationUtils.getDocumentOrientation(activityContext, cameraParamMgr.getRequestedOrientation());
                            int deviceOrientation1 = OrientationUtils.getDeviceOrientation(activityContext);
                            cornerAnalyzer.setOrientation(documentOrientation, deviceOrientation1);
                            mrz1LineAnalyzer.setOrientation(documentOrientation, deviceOrientation1);
                            mrz3LineAnalyzer.setOrientation(documentOrientation, deviceOrientation1);

                            // run corner analyzer first to get corners
                            MiSnapAnalyzerResult cornerAnalyzerResult = cornerAnalyzer.analyze(frame, width, height, colorSpace);
                            // if initial analyzer extracts info, skip later steps
                            processExtraction(cornerAnalyzerResult);

                            if (cornerAnalyzerResult.analyzeSucceeded()) {
                                // We need to delay a bit to avoid blurry images on high-end devices
                                if (lastGoodSnapTimeInMs == 0 && DELAY_BEFORE_ALLOWING_SNAP_IN_MS > 0) {
                                    lastGoodSnapTimeInMs = System.currentTimeMillis();
                                    cornerAnalyzerResult.setErrorCode(IAnalyzeResponse.ANALYZER_FRAME_IS_GOOD_BUT_WE_MUST_WAIT);
                                } else if (System.currentTimeMillis() - lastGoodSnapTimeInMs < DELAY_BEFORE_ALLOWING_SNAP_IN_MS) {
                                    cornerAnalyzerResult.setErrorCode(IAnalyzeResponse.ANALYZER_FRAME_IS_GOOD_BUT_WE_MUST_WAIT);
                                }
                            }

                            // have not yet extracted potential data
                            if (lastExtractedValue == null && !cameraParamMgr.isPassport() && scienceParamMgr.requestOCR()) {
                                int[][] corners = cornerAnalyzerResult.getFourCorners();
                                int[] flatCorners = new int[8];
                                flatCorners[0] = corners[0][0];
                                flatCorners[1] = corners[0][1];
                                flatCorners[2] = corners[1][0];
                                flatCorners[3] = corners[1][1];
                                flatCorners[4] = corners[2][0];
                                flatCorners[5] = corners[2][1];
                                flatCorners[6] = corners[3][0];
                                flatCorners[7] = corners[3][1];

                                List<String> detectedFeatures = documentDetector.detections(frame, width, height, colorSpace, flatCorners, detectorOrientation);

                                for (String feature : detectedFeatures) {
                                    if (feature.equals(MiSnapDocumentDetectorFactory.DETECTED_3LINE_MRZ)) {
                                        MiSnapAnalyzerResult extractionResult = mrz3LineAnalyzer.analyze(frame, width, height, colorSpace);
                                        processExtraction(extractionResult);
                                    } else if (feature.equals(MiSnapDocumentDetectorFactory.DETECTED_1LINE_MRZ)) {
                                        MiSnapAnalyzerResult extractionResult = mrz1LineAnalyzer.analyze(frame, width, height, colorSpace);
                                        processExtraction(extractionResult);
                                    }
                                }
                            }

                            // misnap analyzer is active, send its feedback to UI
                            if (cornerAnalyzerResult.getErrorCode() != IAnalyzeResponse.ANALYZER_IS_NOT_INITIALIZED) {
                                sendResultToUi(cornerAnalyzerResult, width, height, deviceOrientation1,
                                        cameraRotationDegrees, 1 == cameraParamMgr.getUseFrontCamera());
                            }
                            if (cornerAnalyzerResult.analyzeSucceeded()) {
                                if (lastExtractedValue != null || scienceParamMgr.getOCRMode() != ScienceApi.REQUEST_OCR_REQUIRED) {
                                    handleResults(false, cornerAnalyzerResult, width, height,
                                            deviceOrientation1, frame, cameraRotationDegrees, cameraParamMgr.getImageQuality(),
                                            cameraParamMgr.getImageDimensionMax(), 1 == cameraParamMgr.getUseFrontCamera(),
                                            lastExtractedValue);
                                }
                            }
                        }
                    } finally {
                        analyzingInProgress = false;
                    }
                });
            } catch (RejectedExecutionException e) {
                Log.d(TAG, e.toString());
            }
        }
    }

    @Override
    public void handleManuallyCapturedFrame(final byte[] jpegImage, final int width, final int height,
                                            final int deviceOrientation, final int cameraRotationDegrees) {
        if (!executorReady) {
            return;
        }

        try {
            executorService.submit(() -> {
                // run corner analyzer first to get corners
                MiSnapAnalyzerResult cornerAnalyzerResult = cornerAnalyzer.onManualPictureTaken(jpegImage);
                // if initial analyzer extracts info, skip later steps
                processExtraction(cornerAnalyzerResult);

                // have not yet extracted potential data
                if (lastExtractedValue == null && !cameraParamMgr.isPassport() && scienceParamMgr.requestOCR()) {
                    int[][] corners = cornerAnalyzerResult.getFourCorners();
                    int[] flatCorners = new int[8];
                    flatCorners[0] = corners[0][0];
                    flatCorners[1] = corners[0][1];
                    flatCorners[2] = corners[1][0];
                    flatCorners[3] = corners[1][1];
                    flatCorners[4] = corners[2][0];
                    flatCorners[5] = corners[2][1];
                    flatCorners[6] = corners[3][0];
                    flatCorners[7] = corners[3][1];

                    Bitmap bmp = JPEGProcessor.convertJpegToBitmap(jpegImage);
                    ByteBuffer rgba = PreviewFrameConverter.convertBitmapToRgbaBytes(bmp);
                    int width1 = bmp.getWidth();
                    int height1 = bmp.getHeight();
                    bmp.recycle();

                    List<String> detectedFeatures = documentDetector.detections(rgba.array(), width1, height1,
                            HaarCascade.ColorSpace.RGBA, flatCorners, detectorOrientation);

                    for (String feature : detectedFeatures) {
                        if (feature.equals(MiSnapDocumentDetectorFactory.DETECTED_3LINE_MRZ)) {
                            MiSnapAnalyzerResult extractionResult = mrz3LineAnalyzer.onManualPictureTaken(jpegImage);
                            processExtraction(extractionResult);
                        } else if (feature.equals(MiSnapDocumentDetectorFactory.DETECTED_1LINE_MRZ)) {
                            MiSnapAnalyzerResult extractionResult = mrz1LineAnalyzer.onManualPictureTaken(jpegImage);
                            processExtraction(extractionResult);
                        }
                    }
                }

                handleResults(true, cornerAnalyzerResult, width, height, deviceOrientation,
                        jpegImage, cameraRotationDegrees, cameraParamMgr.getImageQuality(),
                        cameraParamMgr.getImageDimensionMax(), 1 == cameraParamMgr.getUseFrontCamera(), lastExtractedValue);
            });
        } catch (RejectedExecutionException e) {
            Log.d(TAG, e.toString());
        }
    }

    public LiveData<MiSnapExtractionControllerResult> getResult() {
        return result;
    }

    public void setDetectorOrientation(int orientation) {
        if (orientation == MiSnapScience.Orientation.PORTRAIT || orientation == MiSnapScience.Orientation.REVERSE_PORTRAIT) {
            detectorOrientation = 90;
        } else {
            detectorOrientation = 0;
        }
    }

    private void processExtraction(MiSnapAnalyzerResult result) {
        if (result.getExtraInfo() instanceof MiSnapAnalyzer.MiSnapAnalyzerOCRExtraInfo) {
            BaseDocument extractedDocument = ((MiSnapAnalyzer.MiSnapAnalyzerOCRExtraInfo) result.getExtraInfo()).documentOcr;

            if (extractedDocument instanceof ExtendedMrz) {
                ExtendedMrz extendedMrz = (ExtendedMrz) extractedDocument;
                if (!extendedMrz.getDocNumber().isEmpty() &&
                        !extendedMrz.getDateOfBirth().isEmpty() &&
                        !extendedMrz.getDateOfExpiration().isEmpty() &&
                        !extendedMrz.getDocType().isEmpty() &&
                        !extendedMrz.getCountry().isEmpty()) {
                    lastExtractedValue = extractedDocument;
                }
            } else if (extractedDocument instanceof BasicMrz && result.getCheckPassed(MiSnapAnalyzerResult.FrameChecks.EXTRACTION_CONFIDENCE)) {
                BasicMrz basicMrz = (BasicMrz) extractedDocument;
                if (basicMrz.getRawData().trim().length() == 30) {
                    lastExtractedValue = extractedDocument;
                }
            }
        }
    }

    private void sendResultToUi(MiSnapAnalyzerResult miSnapAnalyzerResult,
                                int width,
                                int height,
                                int deviceOrientation,
                                int cameraRotationDegrees,
                                boolean useFrontCamera) {
        boolean usePortraitOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT == deviceOrientation ||
                ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT == deviceOrientation;
        int rotationAdjustment = JPEGProcessor.getRotationAngle(Utils.needToRotateFrameBy180Degrees(cameraRotationDegrees),
                usePortraitOrientation, useFrontCamera);
        MiSnapAnalyzerResultsProcessor.updateCorners(miSnapAnalyzerResult,
                rotationAdjustment,
                width,
                height,
                useFrontCamera);
        EventBus.getDefault().post(miSnapAnalyzerResult); // TODO KW:  this is all temporary.  enables workflow to handle hint bubbles.
    }

    private void handleResults(boolean isManualCapture,
                               MiSnapAnalyzerResult miSnapAnalyzerResult,
                               int width,
                               int height,
                               int deviceOrientation,
                               byte[] frameBytes,
                               int cameraRotationDegrees,
                               int imageQuality,
                               int imageDimensionMax,
                               boolean useFrontCamera,
                               @Nullable BaseDocument extraction) {

        // update corners and post to workflow
        boolean usePortraitOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT == deviceOrientation || ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT == deviceOrientation;
        int rotationAdjustment = JPEGProcessor.getRotationAngle(Utils.needToRotateFrameBy180Degrees(cameraRotationDegrees), usePortraitOrientation, useFrontCamera);

        // add mibi/uxp data
        logMibiAndUxp(isManualCapture, deviceOrientation);

        // deinit analyzer; camera is stopped by the manager, for better or worse, via CameraManager.receivedGoodFrame()
        executorReady = false;
        executorService.shutdownNow();
        cornerAnalyzer.deinit();
        mrz3LineAnalyzer.deinit();

        // transform frame and save as jpeg
        byte[] finalFrame;
        if (isManualCapture) {
            EventBus.getDefault().post(miSnapAnalyzerResult); //needed to pass the final warnings to the ControllerFragment
            finalFrame = JPEGProcessor.getFinalJpegFromManuallyCapturedFrame(frameBytes,
                    imageQuality,
                    imageDimensionMax,
                    rotationAdjustment);
        } else {
            finalFrame = JPEGProcessor.getFinalJpegFromPreviewFrame(frameBytes,
                    width,
                    height,
                    imageQuality,
                    imageDimensionMax,
                    rotationAdjustment);
        }

        // call CameraManager.receivedGoodFrame()
        // call MiSnapFragment.processFinalFrameMessage()
        result.postValue(new MiSnapExtractionControllerResult(finalFrame, miSnapAnalyzerResult.getFourCorners(), extraction));

    }

    private void logTheDeviceOrientation(int deviceOrientation) {
        String uxpDeviceOrientation;

        switch (deviceOrientation) {
            case ActivityInfo.SCREEN_ORIENTATION_PORTRAIT:
                uxpDeviceOrientation = UxpConstants.MISNAP_UXP_PORTRAIT_UP;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT:
                uxpDeviceOrientation = UxpConstants.MISNAP_UXP_PORTRAIT_DOWN;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE:
                uxpDeviceOrientation = UxpConstants.MISNAP_UXP_DEVICE_LANDSCAPE_RIGHT;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE:
            default:
                uxpDeviceOrientation = UxpConstants.MISNAP_UXP_DEVICE_LANDSCAPE_LEFT;
        }

        MibiData.getInstance().addUXPEvent(uxpDeviceOrientation);
    }

    private void logMibiAndUxp(boolean isManualCapture, int deviceOrientation) {
        if (!isManualCapture) {
            MibiData.getInstance().addUXPEvent(UxpConstants.MISNAP_UXP_CAPTURE_TIME);
        } else {
            MibiData.getInstance().addUXPEvent(UxpConstants.MISNAP_UXP_CAPTURE_MANUAL);
        }

        logTheDeviceOrientation(deviceOrientation);
    }
}
