package com.miteksystems.misnapextractioncontroller;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.util.Log;
import android.view.ViewGroup;

import androidx.annotation.Nullable;
import androidx.lifecycle.Observer;

import com.miteksystems.misnap.ControllerFragment;
import com.miteksystems.misnap.analyzer.MiSnapAnalyzer;
import com.miteksystems.misnap.camera.MiSnapCamera;
import com.miteksystems.misnap.detector.MiSnapDocumentDetector;
import com.miteksystems.misnap.detector.MiSnapDocumentDetectorFactory;
import com.miteksystems.misnap.events.OnCapturedFrameEvent;
import com.miteksystems.misnap.events.SetCaptureModeEvent;
import com.miteksystems.misnap.params.CameraApi;
import com.miteksystems.misnap.params.CameraParamMgr;
import com.miteksystems.misnap.params.MiSnapApi;
import com.miteksystems.misnap.params.ScienceApi;

import org.greenrobot.eventbus.EventBus;
import org.json.JSONException;
import org.json.JSONObject;

import static com.miteksystems.misnap.params.MiSnapApi.RESULT_ERROR_SDK_STATE_ERROR;

public class MiSnapExtractionFragment extends ControllerFragment {
    private static final String TAG = MiSnapExtractionFragment.class.getName();
    private MiSnapExtractionController controller;
    private MiSnapAnalyzer cornerAnalyzer;
    private MiSnapAnalyzer mrz3LineAnalyzer;
    private MiSnapAnalyzer mrz1LineAnalyzer;
    private MiSnapDocumentDetector documentDetector;

    @Override
    protected void init() {
        super.init();
    }

    @Override
    protected void deinit() {
        super.deinit();
        if (controller != null) {
            controller.end();
            controller = null;
        }

        if (cornerAnalyzer != null) {
            cornerAnalyzer.deinit();
        }

        if (mrz3LineAnalyzer != null) {
            mrz3LineAnalyzer.deinit();
        }

        if (mrz1LineAnalyzer != null) {
            mrz1LineAnalyzer.deinit();
        }
    }


    @Override
    public void initializeController() {
        try {
            //set the surfaceview in the main activity
            MiSnapCamera camera = cameraMgr.getMiSnapCamera();
            if (camera != null) {
                Context appContext = requireActivity().getApplicationContext();
                int documentOrientation = OrientationUtils.getDocumentOrientation(requireActivity(), new CameraParamMgr(miSnapParams).getRequestedOrientation());
                int deviceOrientation = OrientationUtils.getDeviceOrientation(requireActivity());
                cornerAnalyzer = new MiSnapAnalyzer(
                        appContext,
                        miSnapParams,
                        documentOrientation,
                        deviceOrientation,
                        false
                );

                JSONObject customParams = new JSONObject(miSnapParams.toString());
                customParams.put(MiSnapApi.MiSnapDocumentType, MiSnapApi.PARAMETER_DOCTYPE_PDF417);
                mrz3LineAnalyzer = new MiSnapAnalyzer(
                        appContext,
                        replaceDocType(miSnapParams, MiSnapApi.PARAMETER_DOCTYPE_TD1),
                        documentOrientation,
                        deviceOrientation,
                        false
                );
                mrz1LineAnalyzer = new MiSnapAnalyzer(
                        appContext,
                        replaceDocType(miSnapParams, MiSnapApi.PARAMETER_DOCTYPE_1LINE_MRZ),
                        documentOrientation,
                        deviceOrientation,
                        false
                );
                documentDetector = MiSnapDocumentDetectorFactory.getDetector(MiSnapDocumentDetectorFactory.MRZ_DETECTOR);
                cornerAnalyzer.init();
                mrz3LineAnalyzer.init();
                mrz1LineAnalyzer.init();
                controller = new MiSnapExtractionController(
                        requireActivity(),
                        camera,
                        cornerAnalyzer,
                        mrz3LineAnalyzer,
                        mrz1LineAnalyzer,
                        documentDetector,
                        miSnapParams
                );

                // this is needed so that the frame can be returned in return intent
                controller.getResult().observe(this, new Observer<MiSnapExtractionControllerResult>() {
                    @Override
                    public void onChanged(@Nullable MiSnapExtractionControllerResult result) {
                        if (null == result) {
                            Log.w(TAG, "empty result");
                            return;
                        }

                        handleFinalFrame(result);

                        // it's possible for this fragment to go away while analyzer finishes processing a frame
                        if (null != cameraMgr) {
                            cameraMgr.receivedGoodFrame();
                        }
                    }
                });
                controller.setDetectorOrientation(documentOrientation);
                controller.start();

                ((ViewGroup) requireView()).addView(cameraMgr.getSurfaceView());

            } else {
                handleErrorState(RESULT_ERROR_SDK_STATE_ERROR);
            }
        } catch (Exception e) {
            Log.e(TAG, e.toString());
            handleErrorState(RESULT_ERROR_SDK_STATE_ERROR);
        }
    }

    @Override
    public void onEvent(SetCaptureModeEvent event) {
        if (CameraApi.PARAMETER_CAPTURE_MODE_MANUAL == event.mode
                && camParamsMgr.isCurrentModeVideo()) {
            cornerAnalyzer.deinit();
            mrz3LineAnalyzer.deinit();
            mrz1LineAnalyzer.deinit();
        } else if (CameraApi.PARAMETER_CAPTURE_MODE_AUTO == event.mode
                && !camParamsMgr.isCurrentModeVideo()) {
            cornerAnalyzer.init();
            mrz3LineAnalyzer.init();
            mrz1LineAnalyzer.init();
        }

        super.onEvent(event);
    }

    private void handleFinalFrame(MiSnapExtractionControllerResult result) {
        String resultCode;
        if (camParamsMgr.isCurrentModeVideo()) {
            resultCode = MiSnapApi.RESULT_SUCCESS_VIDEO;
            mWarnings.clear();
        } else {
            resultCode = MiSnapApi.RESULT_SUCCESS_STILL;
        }


        Intent returnIntent = buildReturnIntent(Activity.RESULT_OK, result.getFinalFrame(), resultCode, result.getFourCorners());
        if (result.getExtractedData() != null) {
            returnIntent.putExtra(ScienceApi.RESULT_OCR_DATA, result.getExtractedData());
        }
        EventBus.getDefault().post(new OnCapturedFrameEvent(returnIntent));
    }

    private JSONObject replaceDocType(JSONObject params, String docType) throws JSONException {
        JSONObject result = new JSONObject(params.toString());
        result.put(MiSnapApi.MiSnapDocumentType, docType);
        return result;
    }
}
